import sys
import xml.etree.ElementTree as Etree
import re
import socket
import requests
from requests import exceptions

if sys.version_info[0] < 3:
    import tkMessageBox
else:
    from tkinter import messagebox as tkMessageBox


__author__ = 'Mark Baker  email: mark.baker@metoffice.gov.uk'


def get_metar_data(icao_ident):
    """Get the latest METAR for the input ICAO code. The ISO8601 formatted date/time of the observation is returned
    along with the colour state digit code (where 7=BLU, 6=WHT, 5=GRN, 4=YLO1, 3=YLO2, 2=AMB, 1=RED), colour state
    and actual METAR message.

    The xml data feed is provided by the NOAA aviation weather data server. As this provides visibility in statute
    miles, this figure is converted to metres for the purposes of determining colour state.
    :param icao_ident: The identifying ICAO code for the required aerodrome report.
    """
    global root
    socket.setdefaulttimeout(5)

    cloud_heights = [9999]
    sig_cloud_height = 0
    obs_time = ''
    vis_mtrs = 0

    # Query to get the latest METAR xml data
    try:
        root = Etree.fromstring(requests.get('http://aviationweather.gov/adds/dataserver_current/httpparam?dataSource=metars&'
                         'requestType=retrieve&format=xml&stationString=' + icao_ident +
                         '&hoursBeforeNow=2&mostRecent=true', timeout=4, headers={'Connection': 'close'}).content)
    except requests.exceptions.RequestException as e:
        pass
        tkMessageBox.showerror('Communications Error', 'Error retrieving data from NOAA - will retry...')

    # Get the raw METAR message
    for raw_text in root.iter('raw_text'):

        assert isinstance(raw_text.text, str)
        metar_text = raw_text.text

        # Check we have a METAR message for this ICAO ident before proceeding
        if metar_text is not None:

            # Remove text after Q group - this removes any trend text (UK METAR's) which could interfere with
            # current cloud state determination.
            metar_notrend = metar_text.rsplit('Q', 1)[0]

            for observation_time in root.iter('observation_time'):
                obs_time = observation_time.text

            # Note that only certain METAR's contain a colour state reference, hence the presence of this group
            # cannot be relied upon for determining colour state.

            # Ensure colour state will evaluate to BLU if CAVOK present in METAR
            if re.search('CAVOK', metar_notrend):
                vis_mtrs = 9999
                cloud_heights.append(9999)

            # 1613 conversion factor used to ensure metres value never converts to 'just below' colour state limits.
            for visibility_statute_mi in root.iter('visibility_statute_mi'):
                vis_mtrs = round(float(visibility_statute_mi.text) * 1613, 0)  # Convert to metres

            # Check for significant cloud groups and extract the cloud base digits - * 100 to get height in feet.
            # 'FEW' clouds has no effect on colour state so can be ignored.
            if re.search('SCT\d\d\d', metar_notrend):
                cloud_heights_str = re.search(r'SCT\d\d\d', metar_notrend).group()
                cloud_heights.append((int(re.search(r'\d\d\d', cloud_heights_str).group())) * 100)

            if re.search('BKN\d\d\d', metar_notrend):
                cloud_heights_str = re.search(r'BKN\d\d\d', metar_notrend).group()
                cloud_heights.append((int(re.search(r'\d\d\d', cloud_heights_str).group())) * 100)

            if re.search('OVC\d\d\d', metar_notrend):
                cloud_heights_str = re.search(r'OVC\d\d\d', metar_notrend).group()
                cloud_heights.append((int(re.search(r'\d\d\d', cloud_heights_str).group())) * 100)

            if re.search('VV', metar_notrend):
                if re.search('VV///', metar_notrend):
                    cloud_heights.append(0)
                else:
                    cloud_heights_str = re.search('VV\d\d\d', metar_notrend).group()
                    cloud_heights.append((int(re.search(r'\d\d\d', cloud_heights_str).group())) * 100)

            sig_cloud_height = min(cloud_heights)
            colour_state = get_colourstate_nbr(vis_mtrs, sig_cloud_height)

            return obs_time, colour_state[0], colour_state[1], metar_text


def get_taf_data(icao_ident):
    """Get the latest TAF for the input ICAO code. The ISO8601 formatted date/time of the observation is returned
    along with the minimum colour state digit code (where 7=BLU, 6=WHT, 5=GRN, 4=YLO1, 3=YLO2, 2=AMB, 1=RED), colour
    state description and actual TAF message.

    The xml data feed is provided by the NOAA aviation weather data server. As this provides visibility in statute
    miles, this figure is converted to metres for the purposes of determining colour state.
    :param icao_ident: The identifying ICAO code for the required aerodrome report.
    """
    timeout = 10
    socket.setdefaulttimeout(timeout)

    cloud_heights = [9999]
    issue_time = ''
    sig_cloud_height = 0
    vis_mtrs = 9999
    visbilities = [9999.0]

    # Query to get the latest METAR xml data
    try:
        root = Etree.fromstring(requests.get('http://aviationweather.gov/adds/dataserver_current/httpparam?dataSource=tafs&'
                         'requestType=retrieve&format=xml&stationString=' + icao_ident +
                         '&hoursBeforeNow=2&mostRecent=true', timeout=10, headers={'Connection': 'close'}).content)
    except requests.ConnectionError as c:
        errormsg = 'Unable to retrieve data from NOAA: ' + c
        tkMessageBox.showerror('Comms Error', errormsg )

    # Get the raw METAR message
    for raw_text in root.iter('raw_text'):

        assert isinstance(raw_text.text, str)
        taf_text = raw_text.text

        # Check we have a METAR message for this ICAO ident before proceeding
        if taf_text is not None:

            for issue_time in root.iter('issue_time'):
                issue_time = issue_time.text

            # 1613 conversion factor used to ensure metres value never converts to 'just below' colour state limits.
            for visibility_statute_mi in root.iter('visibility_statute_mi'):
                vis_mtrs = round(float(visibility_statute_mi.text) * 1613, 0)  # Convert to metres
                visbilities.append(vis_mtrs)

            # Note that only certain METAR's contain a colour state reference, hence the presence of this group
            # cannot be relied upon for determining colour state.

            # Ensure colour state will evaluate to BLU if CAVOK present in METAR
            if re.search('CAVOK', taf_text):
                vis_mtrs = 9999
                cloud_heights.append(9999)

            # Check for significant cloud groups and extract the cloud base digits - * 100 to get height in feet.
            # 'FEW' clouds has no effect on colour state so can be ignored.
            if re.search('SCT\d\d\d', taf_text):
                cloud_heights_str = re.search(r'SCT\d\d\d', taf_text).group()
                cloud_heights.append((int(re.search(r'\d\d\d', cloud_heights_str).group())) * 100)

            if re.search('BKN\d\d\d', taf_text):
                cloud_heights_str = re.search(r'BKN\d\d\d', taf_text).group()
                cloud_heights.append((int(re.search(r'\d\d\d', cloud_heights_str).group())) * 100)

            if re.search('OVC\d\d\d', taf_text):
                cloud_heights_str = re.search(r'OVC\d\d\d', taf_text).group()
                cloud_heights.append((int(re.search(r'\d\d\d', cloud_heights_str).group())) * 100)

            if re.search('VV', taf_text):
                if re.search('VV///', taf_text):
                    cloud_heights.append(0)
                else:
                    cloud_heights_str = re.search('VV\d\d\d', taf_text).group()
                    cloud_heights.append((int(re.search(r'\d\d\d', cloud_heights_str).group())) * 100)

            min_colour_state = get_colourstate_nbr(min(visbilities), min(cloud_heights))

            return issue_time, min_colour_state[0], min_colour_state[1], taf_text


def get_colourstate_nbr(visibility, cloudbase):
    """Returns the colour state for a given cloud base (in ft) and visibility (in metres). A single digit is also
    returned alongside the colour state to assist in colour state comparisons where 7=BLU, 6=WHT, 5=GRN, 4=YLO1, 3=YLO2,
    2=AMB, 1=RED
    :param cloudbase:
    :param visibility:
    """
    colourstate_nbr = 1
    colourstate = 'RED'

    if visibility < 800 or cloudbase < 200:
        colourstate_nbr = 1  # RED
        colourstate = 'RED'
    elif visibility < 1600 or cloudbase < 300:
        colourstate_nbr = 2  # AMB
        colourstate = 'AMB'
    elif visibility < 2500 or cloudbase < 500:
        colourstate_nbr = 3  # YLO2
        colourstate = 'YLO2'
    elif visibility < 3700 or cloudbase < 700:
        colourstate_nbr = 4  # YLO1
        colourstate = 'YLO1'
    elif visibility < 5000 or cloudbase < 1500:
        colourstate_nbr = 5  # GRN
        colourstate = 'GRN'
    elif visibility < 8000 or cloudbase < 2500:
        colourstate_nbr = 6  # WHT
        colourstate = 'WHT'
    elif visibility > 7999 and cloudbase > 2499:
        colourstate_nbr = 7  # BLU
        colourstate = 'BLU'

    return colourstate_nbr, colourstate


if __name__ == "__main__":
    print('METAR Data= ', get_metar_data('egSH'))



#!/usr/bin/env python
import os
import sys

import PIL
from PIL import ImageTk

if sys.version_info[0] < 3:
    import Tkinter as tk
    import ttk
else:
    import tkinter as tk
    from tkinter import ttk as ttk

__author__ = 'Mark Baker  email: mark.baker@metoffice.gov.uk'


class MainView:
    """Main GUI class responsible for initiating separate window frames for the various monitors."""

    def __init__(self, master):
        self.frame = ttk.Frame(master)
        self.frame.grid(row=0, column=0, padx=5, pady=5)
        self.metar_view = METARview(master)
        self.taf_view = TAFview(master)
        self.controls_view = ControlsView(master)


class METARview:
    """Label window frame containing the widgets for METAR monitoring setup. """

    def __init__(self, root):
        self.arrowup = ImageTk.PhotoImage(file=os.path.join(os.path.dirname(__file__), 'images/arrowup.gif'))
        self.arrowdown = ImageTk.PhotoImage(file=os.path.join(os.path.dirname(__file__), 'images/arrowdown.gif'))
        self.arrowstraight = ImageTk.PhotoImage(file=os.path.join(os.path.dirname(__file__), 'images/arrowstraight.gif'))
        self.grey = ImageTk.PhotoImage(file=os.path.join(os.path.dirname(__file__), 'images/grey.gif'))

        self.frame_metar_monitor = tk.LabelFrame(root, text='Diversion Watch (METAR monitor)')
        self.frame_metar_monitor.grid(row=0, column=0, columnspan=2, sticky=tk.NSEW, padx=5, pady=5)

        self.icao1_label = ttk.Label(self.frame_metar_monitor, text="ICAO :")
        self.icao1_label.grid(row=1, column=0, padx=5, pady=5)

        self.icao1 = tk.StringVar()
        self.icao1_entry = ttk.Entry(self.frame_metar_monitor, textvariable=self.icao1, width=6)
        self.icao1_entry.grid(row=1, column=1, padx=5, pady=5)

        self.icao2_label = ttk.Label(self.frame_metar_monitor, text="ICAO :")
        self.icao2_label.grid(row=2, column=0, padx=5, pady=5)

        self.icao2 = tk.StringVar()
        self.icao2_entry = ttk.Entry(self.frame_metar_monitor, textvariable=self.icao2, width=6)
        self.icao2_entry.grid(row=2, column=1, padx=5, pady=5)

        self.icao3_label = ttk.Label(self.frame_metar_monitor, text="ICAO :")
        self.icao3_label.grid(row=3, column=0, padx=5, pady=5)

        self.icao3 = tk.StringVar()
        self.icao3_entry = ttk.Entry(self.frame_metar_monitor, textvariable=self.icao3, width=6)
        self.icao3_entry.grid(row=3, column=1, padx=5, pady=5)

        self.icao4_label = ttk.Label(self.frame_metar_monitor, text="ICAO :")
        self.icao4_label.grid(row=4, column=0, padx=5, pady=5)

        self.icao4 = tk.StringVar()
        self.icao4_entry = ttk.Entry(self.frame_metar_monitor, textvariable=self.icao4, width=6)
        self.icao4_entry.grid(row=4, column=1, padx=5, pady=5)

        self.prev_last_label = ttk.Label(self.frame_metar_monitor, text='Previous')
        self.prev_last_label.grid(row=0, column=2, padx=5, pady=5)

        self.prev_now_label = ttk.Label(self.frame_metar_monitor, text='Latest')
        self.prev_now_label.grid(row=0, column=3, padx=5, pady=5)

        self.metar1_last_label = ttk.Label(self.frame_metar_monitor, image=self.grey)
        self.metar1_last_label.grid(row=1, column=2, padx=5, pady=5)

        self.metar1_now_label = ttk.Label(self.frame_metar_monitor, image=self.grey)
        self.metar1_now_label.grid(row=1, column=3, padx=5, pady=5)

        self.metar2_last_label = ttk.Label(self.frame_metar_monitor, image=self.grey)
        self.metar2_last_label.grid(row=2, column=2, padx=5, pady=5)

        self.metar2_now_label = ttk.Label(self.frame_metar_monitor, image=self.grey)
        self.metar2_now_label.grid(row=2, column=3, padx=5, pady=5)

        self.metar3_last_label = ttk.Label(self.frame_metar_monitor, image=self.grey)
        self.metar3_last_label.grid(row=3, column=2, padx=5, pady=5)

        self.metar3_now_label = ttk.Label(self.frame_metar_monitor, image=self.grey)
        self.metar3_now_label.grid(row=3, column=3, padx=5, pady=5)

        self.metar4_last_label = ttk.Label(self.frame_metar_monitor, image=self.grey)
        self.metar4_last_label.grid(row=4, column=2, padx=5, pady=5)

        self.metar4_now_label = ttk.Label(self.frame_metar_monitor, image=self.grey)
        self.metar4_now_label.grid(row=4, column=3, padx=5, pady=5)

        self.metar1_arrow_label = ttk.Label(self.frame_metar_monitor, image=self.arrowstraight)
        self.metar1_arrow_label.grid(row=1, column=4, padx=5, pady=5)

        self.metar2_arrow_label = ttk.Label(self.frame_metar_monitor, image=self.arrowstraight)
        self.metar2_arrow_label.grid(row=2, column=4, padx=5, pady=5)

        self.metar3_arrow_label = ttk.Label(self.frame_metar_monitor, image=self.arrowstraight)
        self.metar3_arrow_label.grid(row=3, column=4, padx=5, pady=5)

        self.metar4_arrow_label = ttk.Label(self.frame_metar_monitor, image=self.arrowstraight)
        self.metar4_arrow_label.grid(row=4, column=4, padx=5, pady=5)

        self.latest_metar1_label = ttk.Label(self.frame_metar_monitor, text="Latest METAR/SPECI :")
        self.latest_metar1_label.grid(sticky=tk.W, row=0, column=5, padx=5, pady=5)

        self.latest_metar1 = tk.StringVar()
        self.latest_metar1_result = ttk.Label(self.frame_metar_monitor, textvariable=self.latest_metar1, width=80,
                                              wraplength=480, justify='left', anchor=tk.W)
        self.latest_metar1_result.grid(row=1, column=5, padx=5, pady=5)

        self.latest_metar2 = tk.StringVar()
        self.latest_metar2_result = ttk.Label(self.frame_metar_monitor, textvariable=self.latest_metar2, width=80,
                                              wraplength=480, justify='left', anchor=tk.W)
        self.latest_metar2_result.grid(row=2, column=5, padx=5, pady=5)

        self.latest_metar3 = tk.StringVar()
        self.latest_metar3_result = ttk.Label(self.frame_metar_monitor, textvariable=self.latest_metar3, width=80,
                                              wraplength=480, justify='left', anchor=tk.W)
        self.latest_metar3_result.grid(row=3, column=5, padx=5, pady=5)

        self.latest_metar4 = tk.StringVar()
        self.latest_metar4_result = ttk.Label(self.frame_metar_monitor, textvariable=self.latest_metar4, width=80,
                                              wraplength=480, justify='left', anchor=tk.W)
        self.latest_metar4_result.grid(row=4, column=5, padx=5, pady=5)

    def resource_path(self, relative):
        if hasattr(sys, "_MEIPASS"):
            return os.path.join(sys._MEIPASS, relative)
        return os.path.join(relative)


class TAFview:
    """Label window frame containing the widgets for . """

    def __init__(self, root):
        self.tick = ImageTk.PhotoImage(file=os.path.join(os.path.dirname(__file__), 'images/tick.gif'))
        self.cross = ImageTk.PhotoImage(file=os.path.join(os.path.dirname(__file__), 'images/cross.gif'))
        self.blue = ImageTk.PhotoImage(file=os.path.join(os.path.dirname(__file__), 'images/blue.gif'))
        self.white = ImageTk.PhotoImage(file=os.path.join(os.path.dirname(__file__), 'images/white.gif'))
        self.green = ImageTk.PhotoImage(file=os.path.join(os.path.dirname(__file__),'images/green.gif'))
        self.yellow1 = ImageTk.PhotoImage(file=os.path.join(os.path.dirname(__file__), 'images/yellow1.gif'))
        self.yellow2 = ImageTk.PhotoImage(file=os.path.join(os.path.dirname(__file__), 'images/yellow2.gif'))
        self.amber = ImageTk.PhotoImage(file=os.path.join(os.path.dirname(__file__), 'images/amber.gif'))
        self.red = ImageTk.PhotoImage(file=os.path.join(os.path.dirname(__file__), 'images/red.gif'))
        self.grey = ImageTk.PhotoImage(file=os.path.join(os.path.dirname(__file__), 'images/grey.gif'))

        self.frame_taf_monitor = tk.LabelFrame(root, text='TAF monitor')
        self.frame_taf_monitor.grid(row=1, column=0, sticky=tk.NSEW, padx=5, pady=5)

        self.taf_label = ttk.Label(self.frame_taf_monitor, text="ICAO :")
        self.taf_label.grid(row=1, column=0, padx=5, pady=5)

        self.icao_taf = tk.StringVar()
        self.icao_taf_entry = ttk.Entry(self.frame_taf_monitor, textvariable=self.icao_taf, width=6)
        self.icao_taf_entry.grid(row=1, column=1, padx=5, pady=5)

        self.colour_label = ttk.Label(self.frame_taf_monitor, text='Current')
        self.colour_label.grid(row=0, column=2, padx=0, pady=5)

        self.metar_taf_colour = ttk.Label(self.frame_taf_monitor, image=self.grey)
        self.metar_taf_colour.grid(row=1, column=2, padx=5, pady=5)

        self.taf_status_label = ttk.Label(self.frame_taf_monitor, text='TAF Status')
        self.taf_status_label.grid(row=0, column=3, padx=5, pady=5)

        self.taf_status_label = ttk.Label(self.frame_taf_monitor, text='Latest TAF:')
        self.taf_status_label.grid(sticky=tk.W, row=0, column=5, padx=5, pady=5)

        self.taf_status_result = ttk.Label(self.frame_taf_monitor, image=self.grey)
        self.taf_status_result.grid(row=1, column=3, padx=5, pady=5)

        self.latest_taf = tk.StringVar()
        self.latest_taf_result = ttk.Label(self.frame_taf_monitor, textvariable=self.latest_taf, width=85,
                                           wraplength=510, justify='left', anchor=tk.W)
        self.latest_taf_result.grid(row=1, column=5, padx=5, pady=5)

        self.latest_metar_label = ttk.Label(self.frame_taf_monitor, text="METAR/SPECI:")
        self.latest_metar_label.grid(row=2, column=2, padx=5, pady=5, columnspan=2)

        self.latest_metar = tk.StringVar()
        self.latest_metar_result = ttk.Label(self.frame_taf_monitor, textvariable=self.latest_metar, width=85,
                                             wraplength=510, justify='left', anchor=tk.W)
        self.latest_metar_result.grid(sticky=tk.W, row=2, column=5, padx=5, pady=5, columnspan=6)

    def resource_path(self, relative):
        if hasattr(sys, "_MEIPASS"):
            return os.path.join(sys._MEIPASS, relative)
        return os.path.join(relative)


class ControlsView:
    def __init__(self, root):
        validcmd = (root.register(validate_int), '%d', '%i', '%P', '%s', '%S', '%v', '%V', '%W')

        self.frame_controls = ttk.Frame(root)
        self.frame_controls.grid(row=2, column=0, padx=5, pady=5)

        self.monitor_button = ttk.Button(self.frame_controls, text='Start Monitor', width=15)
        self.monitor_button.grid(sticky=tk.W, row=0, column=0, padx=5, pady=5)

        self.status_label = ttk.Label(self.frame_controls, width=30, text='Status: ')
        self.status_label.grid(sticky=tk.W, row=0, column=1, padx=5, pady=5)

        self.phone_label = ttk.Label(self.frame_controls, text="SMS Alerts (disabled):")
        self.phone_label.grid(sticky=tk.W, row=0, column=2, padx=5, pady=5)

        self.phone_nbr = tk.StringVar()
        self.phone_entry = ttk.Entry(self.frame_controls, textvariable=self.phone_nbr, width=14, validate='key',
                                     validatecommand=validcmd)
        self.phone_entry.grid(sticky=tk.W, row=0, column=3, padx=5, pady=5)

        self.update_data_button = ttk.Button(self.frame_controls, text='Update Now')
        self.update_data_button.grid(sticky=tk.W, row=0, column=4, padx=5, pady=5)

        self.exit_button = ttk.Button(self.frame_controls, text='Exit')
        self.exit_button.grid(sticky=tk.E, row=0, column=5, padx=5, pady=5)


def validate_int(action, index, value_if_allowed,
                 prior_value, text, validation_type, trigger_type, widget_name):
    """

    :param action:
    :param index:
    :param value_if_allowed:
    :param prior_value:
    :param text:
    :param validation_type:
    :param trigger_type:
    :param widget_name:
    :return:
    """
    if not value_if_allowed:
        return True

    if text in '0123456789':
        try:
            int(value_if_allowed)
            return True
        except ValueError:
            return False
    else:
        return False


def validate_float(self, action, index, value_if_allowed,
                   prior_value, text, validation_type, trigger_type, widget_name):
    # action=1 -> insert
    if action == '1':
        if text in '0123456789':
            try:
                float(value_if_allowed)
                return True
            except ValueError:
                return False
        else:
            return False
    else:
        return True



from setuptools import setup, find_packages
from os import path
import codecs
import os
import re

here = path.abspath(path.dirname(__file__))

setup(
    name='metarmonitor',
    version='0.8.6',
    description='Monitor METAR and TAF reports',
    url='www.metpod.co.uk',
    license='MIT',
    author='Mark Baker',
    author_email='mark2182@mac.com',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    install_requires=['pillow', 'requests', 'twilio', 'apscheduler'],
    package_data={
        '': ['images/*.gif'],
    },
    entry_points={
          'gui_scripts': [
               'metarmonitor = metarmonitor.__main__:main'
          ]
    },
)
